package graphDraw;

import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.geom.Ellipse2D;

/**
 * 頂点表示のクラス
 * @author tadaki
 */
public class Node {

    /** 頂点表示の中心 */
    private Point2D.Double center = null;
    /** 頂点表示の半径 */
    private double r = 0.;
    /** 頂点表示の図形 */
    private Ellipse2D.Double ellipse = null;
    /** 頂点の名前 */
    private String name = null;
    /** 頂点表示の色 */
    private Color nodeColor = Color.CYAN;
    /** 頂点名の色 */
    private Color fg = Color.BLACK;

    /**
     * Creates a new instance of Node
     * @param center 中心座標
     * @param r 半径
     * @param name 名前
     */
    public Node(Point2D.Double center, double r, String name) {
        this.center = new Point2D.Double(center.x, center.y);
        this.r = r;
        ellipse = new Ellipse2D.Double(center.x - r, center.y - r,
                2 * r, 2 * r);
        this.name = name;
    }

    /**
     * 頂点の表示
     * @param g 表示に使うGraphicsオブジェクト
     */
    public void draw(Graphics2D g) {
        g.setColor(getNodeColor());
        g.fill(ellipse);
        g.setColor(fg);
        g.drawString(name, (float) center.x, (float) center.y);
    }
    
    /** 設定及び取得メソッド ***********************************************/

    /**
     * @return the nodeColor
     */
    public Color getNodeColor() {
        return nodeColor;
    }

    /**
     * @param nodeColor the nodeColor to set
     */
    public void setNodeColor(Color nodeColor) {
        this.nodeColor = nodeColor;
    }
}
