package graphDraw;

import java.util.List;
import java.awt.*;
import graphLib.Vertex;
import graphLib.Arc;
import graphLib.Graph;

/**
 * グラフを表示するJPanel
 * @author tadaki
 */
public class graphPanel extends javax.swing.JPanel {

    /** 表示されるグラフ */
    private Graph graph = null;
    /** グラムおimage */
    private Image image = null;
    /** 頂点−半径 */
    private double r = 10.;

    /**
     * Creates new form graphPanel
     */
    public graphPanel() {
        initComponents();
    }

    /**
     * imageの作成
     */
    public synchronized void mkImage() {
        if (!isVisible()) {
            return;
        }
        if (graph == null) {
            return;
        }
        Dimension dimension = getPreferredSize();
        image = this.createImage(dimension.width, dimension.height);
        Graphics2D g = (Graphics2D) image.getGraphics();
        g.setColor(this.getBackground());
        g.fillRect(0, 0, dimension.width, dimension.height);
        List<Vertex> vertexes = graph.getVertexes();
        for (Vertex v : vertexes) {
            new Node(v.getPoint(), r, v.toString()).draw(g);
        }

        for (Vertex v : vertexes) {
            if (graph.getArcs(v) != null) {
                for (Arc a : graph.getArcs(v)) {
                    Vertex d = graph.getTail(a);
                    if (!v.equals(d)) {
                        new Arrow(v.getPoint(), d.getPoint(), r,
                                a.toString(), graph.isDirected()).draw(g);
                    }
                }
            }
        }
    }

    /**
     * imageの表示
     * @param g 表示に使うGraphicsオブジェクト
     */
    @Override
    public synchronized void paint(Graphics g) {
        super.paint(g);
        if (image != null) {
            g.drawImage(image, 0, 0, this);
        }
    }

    /** 設定及び取得メソッド ***********************************************/
    /**
     * 表示するグラフの設定
     * @param graph 表示するグラフ
     */
    public void setGraph(graphLib.Graph graph) {
        this.graph = null;
        this.graph = graph;
    }

    /**
     * 頂点半径の設定
     * @param r 頂点半径
     */
    public void setR(double r) {
        this.r = r;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setBackground(new java.awt.Color(204, 255, 204));
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
