package graphLib;

import java.awt.geom.*;

/**
 * 頂点のクラス
 * @author tadaki
 */
public class Vertex implements java.io.Serializable {

    /** 頂点のラベル */
    private String name;
    /** 頂点の位置座標 */
    private Point2D.Double point = null;

    /**
     * Creates a new instance of Vertex
     * @param name 頂点のラベル
     */
    public Vertex(String name) {
        this.name = name;
    }

    /**
     * コピーコンストラクタ
     * @param v 元の頂点
     */
    public Vertex(final Vertex v) {
        this.name = v.name;
        if (v.point != null) {
            this.point = new Point2D.Double(v.point.x, v.point.y);
        }
    }

    /**
     * 頂点のラベルを返す
     * @return 頂点のラベル
     */
    @Override
    public String toString() {
        return getName();
    }

    /** 設定及び取得メソッド *********************************************/
    /**
     * 頂点に設定されている座標を返す
     * @return 頂点に設定されている座標
     */
    public Point2D.Double getPoint() {
        return point;
    }

    /**
     * 頂点に座標を設定する
     * @param p 頂点の座標
     */
    public void setPoint(Point2D.Double p) {
        if (p == null) {
            return;
        }
        point = new Point2D.Double(p.x, p.y);
    }

    /**
     * 頂点に座標を設定する
     * @param x 頂点のx座標
     * @param y 頂点のy座標
     */
    public void setPoint(double x, double y) {
        setPoint(new Point2D.Double(x, y));
    }

    /**
     * 頂点の名前を取得
     * @return  頂点の名前
     */
    public String getName() {
        return name;
    }

    /**
     * 頂点の名前を設定
     * @param name  設定する名前
     */
    public void setName(String name) {
        this.name = name;
    }
}
