/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package xmlUtil;

import java.io.IOException;
import java.io.File;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.DocumentBuilder;

import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.Result;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;

import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author tadaki
 */
public class DOMUtil {

    /** Prints the specified node, then prints all of its children. 
     */
    public static void printDOM(Node node, java.io.BufferedWriter writer)
            throws java.io.IOException {
        int type = node.getNodeType();
        switch (type) {
            // print the document element
            case Node.DOCUMENT_NODE: {
                writer.write("&lt;?xml version=\"1.0\" ?>");
                writer.newLine();
                printDOM(((Document) node).getDocumentElement(), writer);
                break;
            }

            // print element with attributes
            case Node.ELEMENT_NODE: {
                writer.write("&lt;");
                writer.write(node.getNodeName());
                NamedNodeMap attrs = node.getAttributes();
                for (int i = 0; i < attrs.getLength(); i++) {
                    Node attr = attrs.item(i);
                    writer.write(" " + attr.getNodeName().trim()
                            + "=\"" + attr.getNodeValue().trim()
                            + "\"");
                }
                writer.write(">");
                writer.newLine();

                NodeList children = node.getChildNodes();
                if (children != null) {
                    int len = children.getLength();
                    for (int i = 0; i < len; i++) {
                        printDOM(children.item(i), writer);
                    }
                }

                break;
            }

            // handle entity reference nodes
            case Node.ENTITY_REFERENCE_NODE: {
                writer.write("&");
                writer.write(node.getNodeName().trim());
                writer.write(";");
                break;
            }

            // print cdata sections
            case Node.CDATA_SECTION_NODE: {
                writer.write("<![CDATA[");
                writer.write(node.getNodeValue().trim());
                writer.write("]]>");
                break;
            }

// print text
            case Node.TEXT_NODE: {
                writer.write(node.getNodeValue().trim());
                break;
            }

// print processing instruction
            case Node.PROCESSING_INSTRUCTION_NODE: {
                writer.write("<?");
                writer.write(node.getNodeName().trim());
                String data = node.getNodeValue().trim();
                {
                    writer.write(" ");
                    writer.write(data);
                }
                writer.write("?>");
                break;
            }
        }

        if (type == Node.ELEMENT_NODE) {
            writer.newLine();
            writer.write("</");
            writer.write(node.getNodeName().trim());
            writer.write('>');
        }
    }

    /**
     * Parse the XML file and create Document
     * @param fileName
     * @return Document
     */
    public static Document parse(String fileName) {
        return parse(new File(fileName));
    }

    /**
     * Parse the XML file and create Document
     * @param file
     * @return Document
     */
    public static Document parse(File file) {
        Document document = null;
        // Initiate DocumentBuilderFactory
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        // To get a validating parser
        factory.setValidating(false);
        // To get one that understands namespaces
        factory.setNamespaceAware(true);

        try {
            // Get DocumentBuilder
            DocumentBuilder builder = factory.newDocumentBuilder();
            // Parse and load into memory the Document
            document = builder.parse(file);
            return document;

        } catch (SAXParseException spe) {
            // Error generated by the parser
            String st = "\n** Parsing error , line "
                    + spe.getLineNumber() + ", uri " + spe.getSystemId();
            System.err.println(st);
            System.err.println(" " + spe.getMessage());
            // Use the contained exception, if any
            Exception x = spe;
            if (spe.getException() != null) {
                x = spe.getException();
            }
            x.printStackTrace();
        } catch (SAXException sxe) {
            // Error generated during parsing
            Exception x = sxe;
            if (sxe.getException() != null) {
                x = sxe.getException();
            }
            x.printStackTrace();
        } catch (ParserConfigurationException pce) {
            // Parser with specified options can't be built
            pce.printStackTrace();
        } catch (IOException ioe) {
            // I/O error
            ioe.printStackTrace();
        }

        return null;
    }

    /**
     * Parse the XML file and create Document
     * @param is
     * @return Document
     */
    public static Document parse(InputStream is) {
        Document document = null;
        // Initiate DocumentBuilderFactory
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

        // To get a validating parser
        factory.setValidating(false);
        // To get one that understands namespaces
        factory.setNamespaceAware(true);
        // Get DocumentBuilder
        DocumentBuilder builder;
        try {
            builder = factory.newDocumentBuilder();
            document = builder.parse(is);
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(DOMUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (SAXException ex) {
            Logger.getLogger(DOMUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DOMUtil.class.getName()).log(Level.SEVERE, null, ex);
        }
        return document;

    }

    /**
     * This method writes a DOM document to a file
     * @param filename
     * @param document
     */
    public static void writeXmlToFile(String filename, Document document)
            throws TransformerConfigurationException, TransformerException {
        // Prepare the DOM document for writing
        Source source = new DOMSource(document);

        // Prepare the output file
        File file = new File(filename);
        Result result = new StreamResult(file);

        // Write the DOM document to the file
        // Get Transformer
        Transformer xformer =
                TransformerFactory.newInstance().newTransformer();
        // Write to a file
        xformer.transform(source, result);
    }

    /**
     * Count Elements in Document by Tag Name
     * @param tag
     * @param document
     * @return number elements by Tag Name
     */
    public static int countByTagName(String tag, Document document) {
        NodeList list = document.getElementsByTagName(tag);
        return list.getLength();
    }
}
